@@ -1,13 +1,11 @@
+;;; ess-cust.el --- Customize variables for ESS
 
-;; Copyright (C) 1997--2001 A.J. Rossini, Martin Maechler,
-;; Kurt Hornik, Richard M. Heiberger, and Rodney Sparapani.
+;; Copyright (C) 1997--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
-;; Author: A.J. Rossini <rossini@u.washington.edu>
-;; Maintainer: A.J. Rossini <rossini@u.washington.edu>
+;; Original Author: A.J. Rossini <rossini@u.washington.edu>
 ;; Created: 05 June 2000
-;; Modified: $Date: 2002/08/08 09:02:53 $
-;; Version: $Revision: 1.36 $
-;; RCS: $Id: ess-cust.el,v 1.36 2002/08/08 09:02:53 maechler Exp $
+;; Maintainers: ESS-core <ESS-core@stat.math.ethz.ch>
 
 ;; Keywords: editing and process modes.
 
@@ -71,6 +69,11 @@
   :group 'ess
   :prefix "ess-")
 
+(defgroup ess-help nil
+  "ESS: help functions."
+  :group 'ess
+  :prefix "ess-")
+
 (defgroup ess-hooks nil
   "ESS: hooks for customization."
   :group 'ess
@@ -111,7 +114,7 @@
   :group 'ess
   :prefix "ess-")
 
-(defgroup ess-XLS nil
+(defgroup ess-OMG nil
   "ESS: Omegahat."
   :group 'ess
   :prefix "ess-")
@@ -123,10 +126,8 @@
 
 ;; Variables (not user-changeable)
 
-(defcustom ess-version "5.1.24"
-  "Version of ESS currently loaded."
-  :group 'ess
-  :type 'string)
+(defvar ess-version "5.2.3"
+  "Version of ESS currently loaded.")
 
 (defvar no-doc
   "This function is part of ESS, but has not yet been loaded.
@@ -145,19 +146,34 @@ Full documentation will be available after autoloading the function."
 
 ;;*;; Options and Initialization
 
-(defcustom ess-use-menus (featurep 'imenu)
-  "If t, use the menu system with an extra [Imenu] entry.
- Absolutely requires 'imenu."
+;; Menus and pulldowns.
+
+(defcustom ess-funcmenu-use-p (fboundp 'func-menu)
+  "If t, funcmenu is present."
   :group 'ess
-  :type 'boolean)
+  :type  'boolean)
+
+(defcustom ess-speedbar-use-p (fboundp 'speedbar)
+  "If t, speedbar is present."
+  :group 'ess
+  :type  'boolean)
+
+(defcustom ess-imenu-use-p (fboundp 'imenu)
+  "Use imenu facility if it exists.
+This value can be overridden by mode-specific variables, such
+as `ess-imenu-use-S'."
+  :group 'ess
+  :type  'boolean)
+
+;;
 
 (defcustom ess-ask-for-ess-directory t
-  "*If non-nil, the process directory will be requested each time S is run"
+  "*If non-nil, the process directory will be requested each time S is run."
   :group 'ess
   :type 'boolean)
 
 (defcustom ess-ask-about-transfile nil
-  "*If non-nil, asks about a transcript file before running ess"
+  "*If non-nil, asks about a transcript file before running ESS."
   :group 'ess
   :type 'boolean)
 
@@ -176,35 +192,31 @@ Can be changed, e.g., to `R'.  Use `setq-default' if setting it in
 (make-variable-buffer-local 'ess-language)
 (setq-default ess-language "Initial")
 
-(defcustom ess-dialect nil
+(defvar ess-dialect nil
   "String version of the dialect being run for the inferior process.
 This, plus `ess-language', should be able to determine the exact
 version of the statistical package being executed in the particular
 buffer.
 
 Current values could include:
-for `ess-dialect' = S3, S4, S+3, S+4, S+5, S+6, R, XLS, SAS, STA
+for `ess-dialect' = S3, S4, Sp3, Sp4, Sp5, Sp6, R, XLS, SAS, STA
 
-Used to adjust for changes in versions of the program"
-  :group 'ess
-  :type '(choice (const :tag "None"   :value nil)
-		 (const :tag "S3"     :value "S3")
-		 (const :tag "Sp3"    :value "Sp3")
-		 (const :tag "S4"     :value "S4")
-		 (const :tag "Sp4"    :value "Sp4")
-		 (const :tag "Sp5"    :value "Sp5")
-		 (const :tag "Sp6"    :value "Sp6")
-		 (const :tag "XLS"    :value "XLS")
-		 (const :tag "SAS"    :value "SAS")
-		 (const :tag "Runix"  :value "Runix")
-		 (const :tag "Rwin"   :value "Rwin")))
+Used to adjust for changes in versions of the program.")
 
 (make-variable-buffer-local 'ess-dialect)
 ;;(setq-default ess-dialect "Initial-dialect")
-(setq-default ess-dialect nil)		
+(setq-default ess-dialect nil)
 ;;; SJE -- why use "Initial-dialect"?  If we use nil, it matches "None"
 ;;; in the custom choice.
 
+;; (defcustom ess-etc-directory
+;;   (expand-file-name (concat ess-lisp-directory "/../etc/"))
+;;   "*Location of the ESS etc/ directory.
+;; The ESS etc directory stores various auxillary files that are useful
+;; for ESS, such as icons."
+;;   :group 'ess
+;;   :type 'directory)
+
 (defcustom ess-directory-function nil
   "*Function to return the directory that ESS is run from.
 If nil or if the function returns nil then you get `ess-directory'."
@@ -260,7 +272,7 @@ incorrectly, the right things will probably still happen, however."
   :group 'ess-edit
   :type 'boolean)
 
-;;; SJE -- this is set in ess-site.el to be "always", so I changed 
+;;; SJE -- this is set in ess-site.el to be "always", so I changed
 ;;; value t to be "always", so that ess-site.el does not need editing.
 ;;; However, this is a bit messy, and would be nicer if ess-site.el
 ;;; value was t rather than "always".
@@ -309,7 +321,7 @@ nil, and the file is successfully loaded back into S."
 ;;; From ess-mode:
 
 (defcustom ess-mode-silently-save t
-  "*If non-nil, automatically save ESS source buffers before loading"
+  "*If non-nil, automatically save ESS source buffers before loading."
   :group 'ess-edit
   :type 'boolean)
 
@@ -429,6 +441,14 @@ If not number, the statements are indented at open-parenthesis following
 	       (ess-expression-offset . 4)
 	       (ess-else-offset . 0)
 	       (ess-close-brace-offset . 0))
+	  ;; R added ajr 17Feb04 to match "common R" use
+	  (RRR (ess-indent-level . 4)
+	       (ess-continued-statement-offset . 4)
+	       (ess-brace-offset . 0)
+	       (ess-arg-function-offset . 4)
+	       (ess-expression-offset . 4)
+	       (ess-else-offset . 0)
+	       (ess-close-brace-offset . 0))
 	  ;; CLB added rmh 2Nov97 at request of Terry Therneau
 	  (CLB (ess-indent-level . 2)
 	       (ess-continued-statement-offset . 4)
@@ -465,19 +485,23 @@ working directory (i.e. first elt of search list)."
   :type 'directory)
 
 
-(defcustom ess-dump-filename-template (concat (user-login-name) ".%s.S")
-  "*Template for filenames of dumped objects.
-%s is replaced by the object name.
+(defcustom ess-dump-filename-template-proto (concat (user-login-name) ".%s.S")
+  "*Prototype template for filenames of dumped objects.
+The ending `S' is replaced by the current \\[ess-suffix], to give
+\\[ess-dump-filename-template] when an inferior ESS process starts.
 
-This gives filenames like `user.foofun.S', so as not to clash with
+By default, gives filenames like `user.foofun.S', so as not to clash with
 other users if you are using a shared directory. Other alternatives:
 \"%s.S\" ; Don't bother uniquifying if using your own directory(ies)
-\"dump\" ; Always dump to a specific filename. This makes it impossible
+\"dumpdir\"; Always dump to a specific filename. This makes it impossible
          to edit more than one object at a time, though.
 (make-temp-name \"scr.\") ; Another way to uniquify"
+  ;; MM: The last 3-4 lines above suck (I don't understand them) -- FIXME --
+
   :group 'ess-edit
   :type 'string)
 
+
 ;;*;; Hooks
 
 (defcustom ess-mode-hook nil
@@ -503,7 +527,7 @@ Good for evaluating ESS code."
   :type 'hook)
 
 (defcustom inferior-ess-mode-hook nil
-  "*Hook for customizing inferior ess mode.  Called after
+  "*Hook for customizing inferior ESS mode.  Called after
 `inferior-ess-mode' is entered and variables have been initialised."
   :group 'ess-hooks
   :type 'hook)
@@ -513,6 +537,7 @@ Good for evaluating ESS code."
 (put 'inferior-ess-mode 'mode-class 'special)
 ;; FIXME AJR: Should the above be there?  I don't think so!
 ;;	 MM : the functionality should be, right? Move statement to ./ess.el ?
+;;       AJR: No, we should move the statement to ./ess-inf.el
 
 (defcustom ess-help-mode-hook nil
   "Functions to call when entering `ess-help-mode'. "
@@ -532,7 +557,7 @@ Good for evaluating ESS code."
  ; System variables
 
 (defcustom ess-local-process-name nil
-  "The name of the ess process associated with the current buffer."
+  "The name of the ESS process associated with the current buffer."
   :group 'ess
   :type '(choice (const nil) string))
 
@@ -541,19 +566,19 @@ Good for evaluating ESS code."
 
 (defcustom ess-kermit-command "gkermit -T"
     "*Kermit command invoked by `ess-kermit-get' and `ess-kermit-send'."
-    :group 'ess  
+    :group 'ess
     :type  'string
 )
 
 (defcustom ess-kermit-prefix "#"
     "*String files must begin with to use kermit file transfer."
-    :group 'ess  
+    :group 'ess
     :type  'string
 )
 
-(defcustom ess-kermit-remote-directory "$HOME"
+(defcustom ess-kermit-remote-directory "."
     "*Buffer local variable that designates remote directory of file."
-    :group 'ess  
+    :group 'ess
     :type  'string
 )
 
@@ -629,10 +654,91 @@ by `ess-function-template'."
 ;; Keep a copy of your revised ess-site.el to use as a starting point
 ;; for upgrades of ESS.
 
-(defvar inferior-R-program-name
-  (if ess-microsoft-p "Rterm"  "R")
-  "*Program name for invoking an inferior ESS with \\[R].")
+(defcustom inferior-ess-own-frame nil
+  "*Non-nil means that inferior ESS buffers should start in their own frame.
+The parameters of this frame are stored in `inferior-ess-frame-alist'."
+  :group 'ess-proc
+  :type 'boolean)
 
+(defcustom inferior-ess-frame-alist default-frame-alist
+  "*Alist of frame parameters used to create new frames for iESS buffers.
+This defaults to `default-frame-alist' and is used only when
+the variable `inferior-ess-own-frame' is non-nil."
+  :group 'ess-proc
+  :type 'alist)
+
+(defcustom inferior-ess-same-window t
+  "*Non-nil indicates new inferior ESS process appears in current window.
+Otherwise, the new inferior ESS buffer is shown in another window in the
+current frame.  This variable is ignored if `inferior-ess-own-frame' is
+non-nil."
+  :group 'ess-proc
+  :type 'boolean)
+
+(defcustom inferior-R-program-name
+  (if ess-microsoft-p "Rterm"  "R")
+  "*Program name for invoking an inferior ESS with \\[R]."
+  :group 'ess-R
+  :type 'string)
+
+(defcustom inferior-R-args ""
+  "*String of arguments used when starting R.
+These arguments are currently not passed to other versions of R that have
+been created using the variable `ess-r-versions'."
+  :group 'ess-R
+  :type 'string)
+
+(defcustom ess-r-versions '( "R-1" "R-2")
+  "*List of partial strings for versions of R to access within ESS.
+Each string specifies the start of a filename.  If a filename
+beginning with one of these strings is found on `exec-path', a M-x
+command for that version of R is made available.  For example, if the
+file \"R-1.8.1\" is found and this variable includes the string
+\"R-1\", a function called `M-x R-1.8.1' will be available to run that
+version of R.
+If duplicate versions of the same program are found (which happens if
+the same path is listed on `exec-path' more than once), they are
+ignored by calling `ess-uniq-list'."
+  :group 'ess-R
+  :type '(repeat string))
+
+
+(defcustom ess-rterm-versions nil
+"*Construct ess-rterm-versions.  If you have versions of R in
+locations other than in ../../rw*/bin/Rterm.exe, relative to the
+directory in the `exec-path' variable containing your default location
+of Rterm, you will need to redefine this variable with a
+`custom-set-variables' statement in your site-start.el or .emacs
+file."
+  :group 'ess-R
+  :type '(repeat string))
+
+(defcustom ess-SHOME-versions
+  '("c:/progra~1/Insightful/splus62"
+    "c:/progra~1/Insightful/splus61"
+    "c:/progra~1/MathSoft/splus6"
+    "c:/progra~1/spls45se"
+    "c:/progra~1/Insightful/splus62netclient"
+    "c:/progra~1/Insightful/splus62net/server"
+    "c:/progra~1/Insightful/splus61netclient"
+    "c:/progra~1/Insightful/splus61net/server"
+    "c:/progra~1/Insightful/splus6se"
+    "c:/progra~1/Insightful/splus61se"
+    "c:/progra~1/Insightful/splus62se")
+  "*List of possible values of the environment variable SHOME for recent
+releases of S-Plus.  These are the default locations for several
+current and recent releases of S-Plus.  If any of these pathnames
+correspond to a directory on your machine, running the function
+`ess-sqpe-versions-create' will create a function, for example, `M-x
+splus62', that will start the corresponding version Sqpe inside an
+emacs buffer in iESS[S] mode.  If you have versions of S-Plus in
+locations other than these default values, redefine this variable with
+a `custom-set-variables' statement in your site-start.el or .emacs
+file.  The list of functions actually created appears in the *ESS*
+buffer and should appear in the \"ESS / Start Process / Other\"
+menu."
+  :group 'ess-S
+  :type '(repeat string))
 
 (defcustom inferior-S3-program-name "/disk05/s/S"
   "*Program name for invoking an inferior ESS with S3()."
@@ -735,6 +841,12 @@ Use double backslashes if you use the msdos shell."
     :group 'ess-S
     :type 'string))
 
+(defcustom inferior-Splus-args ""
+  "*String of arguments used when starting S.
+These arguments are currently passed only to S+6."
+  :group 'ess-S
+  :type 'string)
+
 (defcustom inferior-S+6-print-command "S_PRINT_COMMAND=gnuclientw.exe"
   "*Destination of print icon in S+6 for Windows Commands window."
   :group 'ess-S
@@ -828,29 +940,33 @@ order for it to work right.  And Emacs is too smart for it."
 (defcustom R-editor
   (if ess-microsoft-p "gnuclient.exe"
     (if (equal system-type 'Apple-Macintosh) nil
-      "emacsclient")) ;; unix
+      (if (featurep 'xemacs) "gnuclient -q" "emacsclient"))) ;; unix
   "*Editor called by R process with 'edit()' command."
-:group 'ess
-:type 'string)
+  :group 'ess
+  :type 'string)
 
-(defcustom R-pager 'nil ; Usually nil is correct as ESS and page() cooperate.
+(defcustom R-pager 'nil	; Usually nil is correct as ESS and page() cooperate.
   "*Pager called by R process with 'page()' command."
-:group 'ess
-:type 'string)
+  :group 'ess
+  :type '(choice (const nil) string))
 
 (defcustom S-editor
   (if ess-microsoft-p "gnuclient.exe"
     (if (equal system-type 'Apple-Macintosh) nil
-      "emacsclient")) ;; unix
+      (if (featurep 'xemacs) "gnuclient -q" "emacsclient"))) ;; unix
   "*Editor called by S process with 'edit()' command."
-:group 'ess
-:type 'string)
+  :group 'ess
+  :type 'string)
 
 (defcustom S-pager
   (if ess-microsoft-p "gnuclientw.exe"
     (if (equal system-type 'Apple-Macintosh) nil
-      "emacsclient")) ;; unix
+      (if (featurep 'xemacs) "gnuclient -q" "emacsclient")))
   "*Pager called by S process with 'page()' command."
+;; Change made to provide a better help(function) experience with
+;; S+6 and xemacs
+;; gnuclient -q will open a buffer with an HTML help file
+;; you can view it with M-x browse-url-of-buffer
 :group 'ess
 :type 'string)
 
@@ -863,10 +979,10 @@ for editing and then to be returned to the process."
 (defcustom ess-pager nil
   "*Pager by which the process sends information to an emacs buffer."
   :group 'ess-proc
-  :type 'string)
+  :type '(choice (const nil) string))
 
 (defcustom inferior-ess-language-start nil
-  "*Initialization commands sent to the ess process."
+  "*Initialization commands sent to the ESS process."
   :group 'ess-proc
   :type 'string)
 
@@ -893,7 +1009,9 @@ for editing and then to be returned to the process."
 (make-variable-buffer-local 'inferior-ess-client-name)
 
 (defcustom inferior-ess-client-command nil
-  "*ddeclient command sent to the ESS program")
+  "*ddeclient command sent to the ESS program."
+  :group 'ess-proc
+  :type '(choice (const nil) string))
 
 (make-variable-buffer-local 'inferior-ess-client-command)
 
@@ -916,11 +1034,9 @@ corresponding program.")
 ;;- 	    ))
 
 
-(defcustom inferior-ess-start-args ""
-  "*String of arguments passed to the ESS process.
-Useful for R and SAS.  This is generic."
-  :group 'ess-proc
-  :type 'string)
+(defvar inferior-ess-start-args ""
+  "String of arguments passed to the ESS process.
+If you wish to pass arguments to a process, see e.g. `inferior-R-args'.")
 
 (defcustom inferior-ess-start-file nil
   "*File dumped into process, if non-nil."
@@ -968,7 +1084,7 @@ in the process buffer."
   :type 'boolean)
 
 (defcustom ess-synchronize-evals nil
-  "*If t, then all evaluations will synchronize with the ess process. This
+  "*If t, then all evaluations will synchronize with the ESS process. This
 means ess-mode will wait for S to dent a prompt before sending the next
 line of code. This allows users of Emacs version 18.57 or less to
 evaluate large regions of code without causing an error.  Users of newer
@@ -1020,15 +1136,15 @@ Might have to:"
 ;; defconst ess-local-process-name now done in S.el
 
 (defcustom ess-process-name-list nil
-  "Alist of active ess processes.")
+  "Alist of active ESS processes.")
 
-;;*;; Inferior ess commands
+;;*;; Inferior ESS commands
 
 (defcustom inferior-ess-load-command "source(\"%s\")\n"
   "Format-string for building the ess command to load a file.
 
 This format string should use %s to substitute a file name and should
-result in an ess expression that will command the inferior ess to load
+result in an ESS expression that will command the inferior ESS to load
 that file."
   :group 'ess-command
   :type 'string)
@@ -1042,7 +1158,7 @@ Use second %s to substitute the dump file name."
   :type 'string)
 
 (defcustom inferior-ess-help-command "help(\"%s\")\n"
-  "Format-string for building the ess command to ask for help on an object.
+  "Format-string for building the ESS command to ask for help on an object.
 
 This format string should use %s to substitute an object name."
   :group 'ess-command
@@ -1056,7 +1172,7 @@ This format string should use %s to substitute an object name."
 
 This format string should use %s to substitute an object name."
   :group 'ess-command
-  :type 'ess-string)
+  :type 'string)
 
 (make-variable-buffer-local 'inferior-ess-exit-command)
 (setq-default inferior-ess-exit-command "q()\n")
@@ -1072,14 +1188,14 @@ Really set in <ess-lang>-customize-alist in ess[dl]-*.el"
 
 
 (defcustom inferior-ess-names-command "names(%s)\n"
-  "Format string for ess command to extract names from an object.
+  "Format string for ESS command to extract names from an object.
 
 %s is replaced by the object name -- usually a list or data frame."
   :group 'ess-command
   :type 'string)
 
 (defcustom inferior-ess-objects-command "ls()\n" ;; others: in (S) or (R)
-  "Format string for ess command to get a list of objects at position %d
+  "Format string for ESS command to get a list of objects at position %d.
 
 Don't include a newline at the end! Used in ess-execute-objects."
   :group 'ess-command
@@ -1106,14 +1222,14 @@ Don't include a newline at the end! Used in ess-execute-objects."
 
 (defcustom ess-change-sp-regexp
   "\\(attach(\\([^)]\\|$\\)\\|detach(\\|collection(\\|library(\\|require(\\|source(\\)"
-  "The regexp for matching the ess commands that change the search path."
+  "The regexp for matching the ESS commands that change the search path."
   :group 'ess-proc
   :type 'regexp)
 
 ;;*;; Process-dependent variables
 
 (defvar ess-search-list nil
-  "Cache of list of directories and objects to search for ess objects.")
+  "Cache of list of directories and objects to search for ESS objects.")
 
 (make-variable-buffer-local 'ess-search-list)
 
@@ -1175,15 +1291,22 @@ See also function `ess-create-object-name-db'.")
 (make-variable-buffer-local 'ess-object-name-db)
 (setq-default ess-object-name-db nil)
 
-(defcustom ess-loop-timeout 500000
+(defcustom ess-S-loop-timeout 1200000
   "Integer specifying how many loops ess-mode will wait for the prompt
-before signaling an error.   This is important for S-PLUS and R, not so
-important for XLispStat.  Increase this, if you have a fast(er) machine."
+before signaling an error. Will be set to `ess-loop-timeout' in the S dialects'
+alists.  Increase this, if you have a fast(er) machine."
   :group 'ess-proc
   :type 'integer)
 
-(make-variable-buffer-local 'ess-loop-timeout)
-;;all dialects set this in their alist: (setq-default ess-loop-timeout 500000)
+(defcustom ess-XLS-loop-timeout 50000
+  "Integer specifying how many loops ess-mode will wait for the prompt
+before signaling an error. Will be set to `ess-loop-timeout' in the XLispStat
+dialects' alists.  Increase this, if you have a fast(er) machine."
+  :group 'ess-proc
+  :type 'integer)
+
+;; NOTA BENE: Other languages/dialect currently set `ess-loop-timeout'
+;;            **directly** in their essd-*.el alist !!
 
 ;;;*;;; Font-lock support
 
@@ -1196,20 +1319,20 @@ If nil, input is in the `font-lock-variable-name-face'."
   :type 'boolean)
 
 (defvar ess-mode-font-lock-keywords
-  '(("<<-\\|<-\\|_\\|->"
+  '(("<<-\\|<-\\|->" ; 2004-01: dropped "_" -- TODO later: for function-name
      . font-lock-reference-face)
     ("\\<\\(TRUE\\|FALSE\\|T\\|F\\|NA\\|NULL\\|Inf\\|NaN\\)\\>"
      . font-lock-type-face)
-    ("\\<\\(library\\|attach\\|detach\\|source\\)\\>"
+    ("\\<\\(library\\|require\\|attach\\|detach\\|source\\)\\>"
      . font-lock-reference-face)
     ("\\<\\(while\\|for\\|in\\|repeat\\|if\\|else\\|switch\\|break\\|next\\|return\\|stop\\|warning\\|function\\)\\>"
-     . font-lock-keyword-face)
-    ("\\s\"?\\(\\(\\sw\\|\\s_\\)+\\(<-\\)?\\)\\s\"?\\s-*\\(<-\\|_\\)\\(\\s-\\|\n\\)*function"
+     . font-lock-keyword-face) ; TODO : drop "_" (here and below)
+    ("\\s\"?\\(\\(\\sw\\|\\s_\\)+\\(<-\\)?\\)\\s\"?\\s-*\\(<-\\|_\\|=\\)\\(\\s-\\|\n\\)*function"
      1 font-lock-function-name-face t))
   "Font-lock patterns used in `ess-mode' buffers.")
 
 (defvar inferior-ess-font-lock-keywords
-  '(("<<-\\|<-\\|_\\|->"
+  '(("<<-\\|<-\\|->"
      . font-lock-reference-face)		; assign
     ("^\\*\\*\\*.*\\*\\*\\*\\s *$"
      . font-lock-comment-face) ; ess-mode msg
@@ -1300,6 +1423,26 @@ If nil, input is in the `font-lock-variable-name-face'."
 
 ;;*;; Variables relating to ess-help-mode
 
+(defcustom ess-help-own-frame nil
+  "*Controls whether ESS help buffers should start in a different frame.
+
+Possible values are:
+   nil: Display help in current frame.
+  'one: All help buffers are shown in one dedicated frame.
+     t: Each help buffer gets its own frame.
+
+The parameters of this frame are stored in `ess-help-frame-alist'.
+See also `inferior-ess-own-frame'."
+  :group 'ess-help
+  :type '(choice (const nil) (const one) (const t)))
+
+(defcustom ess-help-frame-alist special-display-frame-alist
+  "*Alist of frame parameters used to create help frames.
+This defaults to `special-display-frame-alist' and is used only when
+the variable `ess-help-own-frame' is non-nil."
+  :group 'ess-help
+  :type 'alist)
+
 (defconst ess-help-S-sec-regex "^[A-Z. ---]+:$"
   "Reg(ular) Ex(pression) of section headers in help file")
 
@@ -1313,7 +1456,7 @@ If nil, input is in the `font-lock-variable-name-face'."
 
 (defcustom ess-help-kill-bogus-buffers nil
   "*If non-nil, kill ESS help buffers immediately if they are \"bogus\"."
-  :group 'ess
+  :group 'ess-help
   :type 'boolean)
 
 (defvar ess-help-form 'separate-buffer
