@@ -2,17 +2,14 @@
 
 ;; Copyright (C) 1989-1994 Bates, Kademan, Ritter and Smith
 ;; Copyright (C) 1997, A.J. Rossini <rossini@stat.sc.edu>
-;; Copyright (C) 1998--2001	A.J. Rossini <rossini@u.washington.edu>,
-;; Martin Maechler <maechler@stat.math.ethz.ch>, Kurt Hornik < >,
-;; and Richard M. Heiberger <rmh@temple.edu>.
+;; Copyright (C) 1998--2001 A.J. Rossini, Martin Maechler, Kurt Hornik and
+;;	Richard M. Heiberger <rmh@temple.edu>.
+;; Copyright (C) 2001--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
-;; Author: David Smith <dsmith@stats.adelaide.edu.au>
-;; Maintainers: A.J. Rossini <rossini@u.washington.edu>,
-;;              Martin Maechler <maechler@stat.math.ethz.ch>
+;; Original Author: David Smith <dsmith@stats.adelaide.edu.au>
 ;; Created: 7 Jan 1994
-;; Modified: $Date: 2002/01/28 08:36:32 $
-;; Version: $Revision: 5.17 $
-;; RCS: $Id: ess-help.el,v 5.17 2002/01/28 08:36:32 maechler Exp $
+;; Maintainers: ESS-core <ESS-core@stat.math.ethz.ch>
 
 ;; This file is part of ESS
 
@@ -128,16 +125,16 @@ Uses the variable `inferior-ess-help-command' for the actual help command."
 			  "](" object ")*"))
 	 (old-hb-p	(get-buffer hb-name))
 	 (curr-win-mode major-mode)
-	 (tbuffer 	(get-buffer-create hb-name))
+	 (tbuffer	(get-buffer-create hb-name))
 	 (curr-help-command		inferior-ess-help-command)
 	 ;;-- pass the buffer-local 'ess-help-sec-..'  to the ess-help buffer:
-	 (curr-help-sec-regex	        ess-help-sec-regex)
+	 (curr-help-sec-regex		ess-help-sec-regex)
 	 (curr-help-sec-keys-alist	ess-help-sec-keys-alist)
 	 (curr-help-syntax-table	(syntax-table))
 	 (alist		ess-local-customize-alist))
 
     (set-buffer tbuffer)
-    (ess-setq-vars-local (eval alist) (current-buffer))
+    (ess-setq-vars-local (eval alist))
     (setq ess-help-sec-regex	  curr-help-sec-regex)
     (setq ess-help-sec-keys-alist curr-help-sec-keys-alist)
     ;; see above, do same for inferior-ess-help-command... (i.e. remove
@@ -157,6 +154,7 @@ Uses the variable `inferior-ess-help-command' for the actual help command."
 	  (ess-command (format curr-help-command object) tbuffer)
 	  ;; was inferior-ess-help-command
 
+	  (ess-help-underline)
 	  ;; Stata is clean, so we get a big BARF from this.
 	  (if (not (string= ess-language "STA"))
 	      (ess-nuke-help-bs))
@@ -187,15 +185,45 @@ Uses the variable `inferior-ess-help-command' for the actual help command."
 	  ;; else : show it
 
 	  ;;dbg (ess-write-to-dribble-buffer
-	  ;;dbg  (format "(ess-help %s before switch-to..\n" hb-name)
-	  (if (eq curr-win-mode 'ess-help-mode)
-	      (switch-to-buffer tbuffer)
-	    (ess-display-temp-buffer tbuffer))
+	  ;;dbg	 (format "(ess-help %s before switch-to..\n" hb-name)
+	  (let ((special-display-regexps
+		 (if ess-help-own-frame '(".") nil))
+		(special-display-frame-alist ess-help-frame-alist)
+		(special-display-function
+		 (if (eq ess-help-own-frame 'one)
+		     'ess-help-own-frame
+		   special-display-function)))
+	    (if (eq curr-win-mode 'ess-help-mode)
+		(if ess-help-own-frame
+		    (pop-to-buffer tbuffer)
+		  (switch-to-buffer tbuffer))
+	      (ess-display-temp-buffer tbuffer)))
 	  (if curr-help-syntax-table
 	      (set-syntax-table curr-help-syntax-table))
 	  (set-buffer-modified-p 'nil)
 	  (toggle-read-only t))))))
 
+(defvar ess-help-frame nil
+  "Stores the frame used for displaying R help buffers.")
+
+(defun	ess-help-own-frame (buffer &rest ignore)
+  "Put all ESS help buffers into `ess-help-frame'."
+  ;; SJE: Code adapted from Kevin Rodgers.
+  (if (frame-live-p ess-help-frame)
+      (progn
+	(or (frame-visible-p ess-help-frame)
+	    (make-frame-visible ess-help-frame))
+	(raise-frame ess-help-frame)
+	(select-frame ess-help-frame)
+	(switch-to-buffer buffer)
+	(selected-window))
+    ;; else
+    (let ((window (special-display-popup-frame buffer)))
+      (set-window-dedicated-p window nil)
+      (setq ess-help-frame (window-frame window))
+      window)))
+
+
 
 ;;; THIS WORKS!
 ;;(require 'w3)
@@ -321,9 +349,9 @@ Other keybindings are as follows:
 ;;*;; User commands defined in ESS help mode
 
 (defun ess-skip-to-help-section nil
-  "Jump to a section heading of a help buffer.  The section selected
+  "Jump to a section heading of a help buffer.	The section selected
 is determined by the command letter used to invoke the command, as
-indicated by `ess-help-sec-keys-alist'.  Use \\[ess-describe-sec-map]
+indicated by `ess-help-sec-keys-alist'.	 Use \\[ess-describe-sec-map]
 to see which keystrokes find which sections."
   (interactive)
   (let ((old-point (point))
@@ -462,6 +490,14 @@ Stata or XLispStat for additional information."
   (skip-chars-forward "\n")
   (delete-region (point-min) (point)))
 
+(defun ess-help-underline ()
+  "Replace ^_H codes with underline face."
+  (save-excursion
+    (goto-char (point-min))
+    (while (search-forward "_" nil t)
+    (backward-delete-char 2)
+    (put-text-property (point) (1+ (point)) 'face 'underline))))
+
 ;;*;; Link to Info
 
 (defun ess-goto-info (node)
