@@ -1,16 +1,14 @@
 ;;; ess-inf.el --- Support for running S as an inferior Emacs process
 
 ;; Copyright (C) 1989-1994 Bates, Kademan, Ritter and Smith
-;; Copyright (C) 1997-2001 A.J. Rossini <rossini@u.washington.edu>,
-;; Martin Maechler <maechler@stat.math.ethz.ch>.
+;; Copyright (C) 1997-1999 A.J. Rossini <rossini@u.washington.edu>,
+;;	Martin Maechler <maechler@stat.math.ethz.ch>.
+;; Copyright (C) 2000--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
-;; Author: David Smith <dsmith@stats.adelaide.edu.au>
-;;         (now: dsmith@insightful.com)
-;; Maintainer: A.J. Rossini <rossini@u.washington.edu>
+;; Original Author: David Smith <dsmith@stats.adelaide.edu.au>
 ;; Created: 7 Jan 1994
-;; Modified: $Date: 2002/06/20 21:52:06 $
-;; Version: $Revision: 5.78 $
-;; RCS: $Id: ess-inf.el,v 5.78 2002/06/20 21:52:06 rsparapa Exp $
+;; Maintainers: ESS-core <ESS-core@stat.math.ethz.ch>
 
 ;; This file is part of ESS
 
@@ -115,8 +113,10 @@ accompany the call for `inferior-ess-program'.
   ;; Couldn't we rather set all the default values or Local values now ?
   ;;>>> (ess-setq-vars-default ess-customize-alist (current-buffer))
   ;;>>> (ess-setq-vars-local   ess-customize-alist (current-buffer))
-  ;;; AJR sez: I think we should set them later.  I don't want to nuke if
+  ;;; AJR sez: I think we should set them later; don't want to nuke if
   ;;; I don't have to.
+  ;;- MM: We shouldn't have to use  ess-setq-vars-default _at all_ ;
+  ;;      only do the buffer local ...-vars-local ones
   (let ((temp-ess-dialect (eval (cdr (assoc 'ess-dialect
 				       ess-customize-alist))))
 	(temp-ess-lang (eval (cdr (assoc 'ess-language
@@ -124,16 +124,25 @@ accompany the call for `inferior-ess-program'.
     (save-excursion
       ;;- Is this needed? (no, but it's useful to see them there [MM])
       (set-buffer ess-dribble-buffer)
-      (ess-setq-vars-default ess-customize-alist (current-buffer))
+      ;; Hack to work around the following "default" (global) setting of vars:
+      ;; make sure our comint-... hack doesn't affect anything else
+      (make-variable-buffer-local 'comint-use-prompt-regexp-instead-of-fields)
+      ;; now the abomination:
+      (ess-setq-vars-default ess-customize-alist)
+
+      (setq-default comint-use-prompt-regexp-instead-of-fields nil); re set HACK!
       ;;>> Doesn't set ess-language,
       ;;>> => comint-input-sender is not set to 'ess-input-  ==> no input echo!
       ;;>> => that's why things fail:
       ;;>> (ess-setq-vars-local ess-customize-alist (current-buffer))
       ;;		 ======
-      (setq temp-ess-dialect
-	    (eval(cdr(assoc 'ess-dialect ess-customize-alist))))
-      (setq temp-ess-lang
-	    (eval(cdr(assoc 'ess-language ess-customize-alist)))))
+      ;;MM --- why set them here again when already "(let ..)" above ?
+      ;;   --> trying without (comment the following 4 lines):
+      ;; (setq temp-ess-dialect
+      ;;    (eval(cdr(assoc 'ess-dialect ess-customize-alist))))
+      ;; (setq temp-ess-lang
+      ;;    (eval(cdr(assoc 'ess-language ess-customize-alist))))
+      )
 
     ;; run hooks now, to overwrite the above!
     (run-hooks 'ess-pre-run-hook)
@@ -141,8 +150,8 @@ accompany the call for `inferior-ess-program'.
      (format "(inf-ess 1): lang=%s, dialect=%s, tmp-dialect=%s, buf=%s\n"
 	     ess-language ess-dialect temp-ess-dialect (current-buffer)))
     (let* ((process-environment process-environment)
-           (defdir (or (and ess-directory-function (funcall ess-directory-function))
-                       ess-directory default-directory))
+	   (defdir (or (and ess-directory-function (funcall ess-directory-function))
+		       ess-directory default-directory))
 	   (temp-dialect temp-ess-dialect)
 	   (temp-lang temp-ess-lang)
 	   (procname (let ((ntry 0) ;; find a non-existent process
@@ -170,18 +179,18 @@ accompany the call for `inferior-ess-program'.
 	     (memq major-mode '(inferior-ess-mode)))
 	(setq startdir
 	      (if ess-ask-for-ess-directory
-                  (ess-get-directory (directory-file-name defdir))
-                  defdir))
+		  (ess-get-directory (directory-file-name defdir))
+		  defdir))
 	(setq buf (current-buffer))
 	(ess-write-to-dribble-buffer
-         (format "(inferior-ess) Method #1 start=%s buf=%s\n" startdir buf)))
+	 (format "(inferior-ess) Method #1 start=%s buf=%s\n" startdir buf)))
 
        ;;  Not an ESS buffer yet
        ((and (not buf)
 	     (get-buffer buf-name-str))
 	(setq buf (get-buffer buf-name-str))
 	(ess-write-to-dribble-buffer
-         (format "(inferior-ess) Method #2 buf=%s\n" buf)))
+	 (format "(inferior-ess) Method #2 buf=%s\n" buf)))
 
        ;; Ask for transcript file and startdir
        ;; FIXME -- this should be in ess-get-transfile
@@ -190,8 +199,8 @@ accompany the call for `inferior-ess-program'.
        ((not buf)
 	(setq startdir
 	      (if ess-ask-for-ess-directory
-                  (ess-get-directory (directory-file-name defdir))
-                  defdir))
+		  (ess-get-directory (directory-file-name defdir))
+		  defdir))
 	(if ess-ask-about-transfile
 	    (let ((transfilename (read-file-name
 				  "Use transcript file (default none):"
@@ -202,11 +211,11 @@ accompany the call for `inferior-ess-program'.
 					       transfilename)))))
 	  (setq buf (get-buffer-create buf-name-str)))
 	(ess-write-to-dribble-buffer
-         (format "(inferior-ess) Method #3 start=%s buf=%s\n" startdir buf))))
+	 (format "(inferior-ess) Method #3 start=%s buf=%s\n" startdir buf))))
 
       (set-buffer buf)
       ;; Now that we have the buffer, set buffer-local variables.
-      (ess-setq-vars-local ess-customize-alist buf)
+      (ess-setq-vars-local ess-customize-alist); buf)
       (if ess-start-args (setq inferior-ess-start-args ess-start-args))
       ;; Was:  if not, set to null.
       ;;(setq inferior-ess-start-args "")) ;; AJR: Errors with XLS?
@@ -237,7 +246,7 @@ accompany the call for `inferior-ess-program'.
 		    (concat "." ess-dialect "history"))
       ;; initialize.
       (let ((ess-directory (if startdir default-directory ess-directory)))
-        (ess-multi procname buf inferior-ess-start-args)))))
+	(ess-multi procname buf inferior-ess-start-args)))))
 
 
 ;;; A note on multiple processes: the following variables
@@ -263,10 +272,14 @@ start up a new process, using NAME and BUFFER (which is needed if
 there is no process NAME)."
 
   (let* ((proc-name name)
+	 (special-display-regexps nil)
+	 (special-display-frame-alist inferior-ess-frame-alist)
 	 (proc (get-process proc-name)))
+    (if inferior-ess-own-frame
+	(setq special-display-regexps '(".")))
     ;; If ESS process NAME is running, switch to it
     (if (and proc (comint-check-proc (process-buffer proc)))
-	(switch-to-buffer (process-buffer proc))
+	(pop-to-buffer (process-buffer proc))
       ;; Otherwise, crank up a new process
       (let* ((symbol-string
 	      (concat "inferior-" inferior-ess-program "-args"))
@@ -326,7 +339,10 @@ there is no process NAME)."
 	;; Get search list when needed
 	(setq ess-sp-change t)
 	(run-hooks 'ess-post-run-hook))
-      (switch-to-buffer (process-buffer (get-process proc-name))))))
+      (if inferior-ess-same-window
+	  (switch-to-buffer (process-buffer (get-process proc-name)))
+	(pop-to-buffer (process-buffer (get-process proc-name)))))))
+
 
 (defun ess-process-sentinel (proc message)
   "Sentinel for use with ESS processes.
@@ -399,7 +415,7 @@ This was rewritten by KH in April 1996."
 
 (defun ess-get-directory (default)
   (ess-prompt-for-directory
-        default
+	default
 	(format "ESS [%s(%s): %s] starting data directory? "
 		ess-language ess-dialect inferior-ess-program)))
 
@@ -408,9 +424,14 @@ This was rewritten by KH in April 1996."
   (let ((the-dir
 	 (expand-file-name
 	  (file-name-as-directory
-	   (read-file-name prompt
-			   (file-name-as-directory default)
-			   (file-name-as-directory default) t nil)))))
+	   (if (fboundp 'read-directory-name)
+	       ;; use XEmacs' read-directory-name if exists.
+	       (read-directory-name prompt
+				    (file-name-as-directory default)
+				    (file-name-as-directory default) t nil)
+	     (read-file-name prompt
+			     (file-name-as-directory default)
+			     (file-name-as-directory default) t nil))))))
     (if (file-directory-p the-dir) nil
       (error "%s is not a valid directory" the-dir))
     the-dir))
@@ -488,8 +509,9 @@ Returns the name of the process, or nil if the current buffer has none."
 
 (defun ess-request-a-process (message &optional noswitch)
   "Ask for a process, and make it the current ESS process.
-Also switches to the process buffer, if second arg NOSWITCH (prefix)
-is non-nil.  Returns the name of the selected process."
+Also switches to the process buffer unless NOSWITCH is non-nil.  Interactively,
+NOSWITCH can be set by giving a prefix argument.
+Returns the name of the selected process."
   (interactive
    (list "Switch to which ESS process? " current-prefix-arg)) ;prefix sets 'noswitch
   (update-ess-process-name-list)
@@ -508,12 +530,14 @@ is non-nil.  Returns the name of the selected process."
     (save-excursion
       (set-buffer (process-buffer (get-process proc)))
       (ess-make-buffer-current))
-    (if noswitch nil (switch-to-buffer (process-buffer (get-process proc))))
+    (if noswitch
+	nil
+      (ess-show-buffer (buffer-name (process-buffer (get-process proc))) t))
     proc))
 
 (defun ess-force-buffer-current (prompt &optional force)
   "Make sure the current buffer is attached to an ESS process.
-If not, or  FORCE (prefix argument) is non-nil,
+If not, or FORCE (prefix argument) is non-nil,
 prompt for a process name with PROMPT.
 `ess-local-process-name' is set to the name of the process selected.
 `ess-dialect' is set to the dialect associated with the process selected."
@@ -543,13 +567,18 @@ prompt for a process name with PROMPT.
 
 (defun ess-switch-to-ESS (eob-p)
   "Switch to the current inferior ESS process buffer.
-With (prefix) EOB-P non-nil, positions cursor at end of buffer."
+With (prefix) EOB-P non-nil, positions cursor at end of buffer.
+This function should follow the description in `ess-show-buffer'
+for showing the iESS buffer, except that the iESS buffer is also
+made current."
   (interactive "P")
   (ess-make-buffer-current)
   (if (and ess-current-process-name (get-process ess-current-process-name))
       (progn
-	(pop-to-buffer
-	 (process-buffer (get-process ess-current-process-name)))
+	;; Display the buffer, but don't select it yet.
+	(ess-show-buffer
+	 (buffer-name (process-buffer (get-process ess-current-process-name)))
+	 t)
 	(if eob-p (goto-char (point-max))))
     (message "No inferior ESS process")
     (ding)))
@@ -579,6 +608,98 @@ With (prefix) EOB-P non-nil, positions cursor at end of buffer."
   (if (eq (length ess-process-name-list) 0)
       (setq ess-current-process-name nil)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; ess-show-buffer
+;; Something like this almost works, but problems with XEmacs and Emacs
+;; differing implementations of the args to display-buffer make this
+;; too tough to pursue.  The longer version below works.
+;; (defun ess-show-buffer (buf)
+;;   "Display the buffer BUF, a string, but do not select it.
+;; Returns the window corresponding to the buffer."
+;;   ;; On XEmacs, I get an error if third arg to display-buffer is t and
+;;   ;; the BUF is in another frame.  Emacs does not have this problem.
+;;   (if (featurep 'xemacs)
+;;       (display-buffer buf nil (get-frame-for-buffer buf))
+;;     (display-buffer buf nil t)))
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defun ess-show-buffer (buf &optional visit)
+  "Ensure the ESS buffer BUF is visible.
+The buffer, specified as a string, is typically an iESS (e.g. *R*) buffer.
+
+This handles several cases:
+
+1. If BUF is visible in the current frame, nothing is done.
+2. If BUF is visible in another frame, then we ensure that frame is
+visible (it may have been iconified).
+3. If buffer is not visible in any frame, simply show it in another window
+in the current frame.
+
+Iff VISIT is non-nil, as well as making BUF visible, we also select it
+as the current buffer."
+  (let ( (frame))
+    (if (ess-buffer-visible-this-frame buf)
+	;;1. Nothing to do, BUF visible in this frame; just return window
+	;; where this buffer is.
+	t
+
+      ;; 2. Maybe BUF visible in another frame.
+      (setq frame (ess-buffer-visible-other-frame buf))
+      (if frame
+	  ;; BUF is visible in frame, so just check frame is raised.
+	  (if (not (eq (frame-visible-p frame) t))
+	      ;; frame is not yet visible, so raise it.
+	      (raise-frame frame))
+	;; 3. else BUF not visible in any frame, so show it (but do
+	;; not select it) in another window in current frame.
+	(display-buffer buf)))
+    ;; At this stage, the buffer should now be visible on screen,
+    ;; although it won't have been made current.
+    (if visit
+	(progn
+	  ;; Need to select the buffer.
+	  ;;
+	  ;; First of all, check case 2 if buffer is in another frame
+	  ;; but that frame may not be selected.
+	  (if (and frame (not (featurep 'xemacs)))
+	      (progn
+		;; need to select the frame
+		(select-frame frame)
+		;; reposition mouse to make frame active.
+		(set-mouse-position (selected-frame) (1- (frame-width)) 0)))
+	  (select-window (get-buffer-window buf 0))))))
+
+
+;; The next few functions are copied from my (SJE) iswitchb library.
+(defun ess-get-bufname (win)
+  "Used by `ess-get-buffers-in-frames' to walk through all windows."
+  (let ((buf (buffer-name (window-buffer win))))
+    (if (not (member buf ess-bufs-in-frame))
+	;; Only add buf if it is not already in list.
+	;; This prevents same buf in two different windows being
+	;; put into the list twice.
+	(setq ess-bufs-in-frame
+	      (cons buf ess-bufs-in-frame)))))
+
+(defun ess-get-buffers-in-frames (&optional current)
+ "Return the list of buffers that are visible in the current frame.
+If optional argument CURRENT is given, restrict searching to the
+current frame, rather than all frames."
+ (let ((ess-bufs-in-frame nil))
+   (walk-windows 'ess-get-bufname nil (if current nil 0))
+   ess-bufs-in-frame))
+
+(defun ess-buffer-visible-this-frame (buf)
+ "Return t if BUF is visible in current frame."
+ (member buf (ess-get-buffers-in-frames t)))
+
+(defun ess-buffer-visible-other-frame (buf)
+ "Return t if BUF is visible in another frame.
+Assumes that buffer has not already been in found in current frame."
+ (if (member buf (ess-get-buffers-in-frames))
+     (window-frame (get-buffer-window buf 0))
+   nil))
+
+
  ; Functions for evaluating code
 
 (defun ess-ddeclient-p ()
@@ -830,11 +951,13 @@ EOB is non-nil go to end of ESS process buffer after evaluation.  If optional
     (goto-char (marker-position (process-mark sprocess)))
     (if eob
 	(progn
-	  (set-buffer cbuffer)
-	  (switch-to-buffer-other-window sbuffer)
-	  (goto-char (point-max))
-	  (switch-to-buffer-other-window cbuffer))
-      (set-buffer cbuffer))))
+	  (ess-show-buffer (buffer-name sbuffer) nil)
+	  ;; Once SBUFFER is visible, we can then move the point in that
+	  ;; window to the end of the buffer.
+	  (set-window-point (get-buffer-window sbuffer t)
+			    (with-current-buffer sbuffer (point-max))))
+      (set-buffer cbuffer))
+    ))
 
 ;;;*;;; Evaluate only
 
@@ -1003,7 +1126,6 @@ process buffer. Arg has same meaning as for `ess-eval-region'."
 		      (buffer-file-name))
 		 (expand-file-name
 		  (read-file-name "Load S file: " nil nil t)))))
-  (require 'ess-inf)
   (ess-make-buffer-current)
   (let ((source-buffer (get-file-buffer filename)))
     (if (ess-check-source filename)
@@ -1071,10 +1193,13 @@ process buffer. Arg has same meaning as for `ess-eval-region'."
 	 ;; Code for GNU Emacs
 	 (setq inferior-ess-mode-map (cons 'keymap comint-mode-map))))
 
-  ;; Use syntax valid *both* for GNU emacs and Xemacs :
+  ;; Use syntax valid *both* for GNU emacs and XEmacs :
   (define-key inferior-ess-mode-map "\r"       'inferior-ess-send-input)
+  (define-key inferior-ess-mode-map "\C-a"     'comint-bol)
   (define-key inferior-ess-mode-map "\M-\r"    'ess-transcript-send-command-and-move)
   (define-key inferior-ess-mode-map "\C-c\C-l" 'ess-load-file)
+  ;; the above OVERRIDES  comint-dynamic-list-input-ring --> re-assign:
+  (define-key inferior-ess-mode-map "\C-c\M-l" 'comint-dynamic-list-input-ring)
   (define-key inferior-ess-mode-map "\C-c`"    'ess-parse-errors)
   (define-key inferior-ess-mode-map "\C-c\C-d" 'ess-dump-object-into-edit-buffer)
   (define-key inferior-ess-mode-map "\C-c\C-v" 'ess-display-help-on-object)
@@ -1222,6 +1347,8 @@ to continue it."
 
   (make-local-variable 'comint-process-echoes)
   (make-local-variable 'comint-input-sender)
+  (make-local-variable 'process-connection-type)
+  (setq process-connection-type t)
 
   ;; Configuration for SAS/XLispStat input handling
   ;; We set comint-process-echoes to t because inferior-ess-input-sender
@@ -1275,7 +1402,7 @@ to continue it."
   (setq font-lock-defaults
 	'(inferior-ess-font-lock-keywords nil nil ((?' . "."))))
 
-  (ess-setq-vars-local ess-customize-alist (current-buffer))
+  (ess-setq-vars-local ess-customize-alist); (current-buffer))
 
   (ess-write-to-dribble-buffer
    (format "(i-ess 3): curr-buf=%s, comint..echo=%s, comint..sender=%s,\n"
@@ -1335,11 +1462,11 @@ to continue it."
 ;;>
 ;;> As promised, here is a quick hack:
 ;;  ___hack much improved by MM___ , both help(.) and ?... now work
-;;
-;; rmh: catch page() just like we catch help()
+;; FIXME: Note that  '??' nicely works in *R*, but
+;;        'type ? topic' doesn't use ess-help {but display in *R*}
 (defconst inferior-R-1-input-help (format "help *(%s)" ess-help-arg-regexp))
 (defconst inferior-R-2-input-help (format "^ *\\? *%s" ess-help-arg-regexp))
-(defconst inferior-R-page         (format "page *(%s)" ess-help-arg-regexp))
+(defconst inferior-R-page         (format "^ *page *(%s)" ess-help-arg-regexp))
 
 (defun inferior-R-input-sender (proc string)
   ;; REALLY only for debugging: this S_L_O_W_S D_O_W_N   [here AND below]
@@ -1359,7 +1486,7 @@ to continue it."
 		(insert-before-markers string)) ;; emacs 21.0.105 and older
 	    (delete-backward-char 1))           ;; emacs 21.0.106 and newer
 	  (if page-string
-	      (progn	      
+	      (progn
 		(ess-command (concat string2 "\n")
 			     (get-buffer-create (concat string2 ".rt")))
 		(ess-eval-linewise "\n")
@@ -1533,7 +1660,6 @@ to the command if BUFF is not given.)"
     (if (yes-or-no-p (format "Really quit ESS process %s? " sprocess))
 	(progn   ;;;;previouslyl (save-excursion
 	  (ess-cleanup)
-	  (ess-switch-to-ESS nil)
 	  (goto-char (marker-position (process-mark sprocess)))
 	  (insert inferior-ess-exit-command)
 	  (if (string-equal ess-dialect "R")
@@ -1570,18 +1696,19 @@ before you quit.  It is run automatically by \\[ess-quit]."
 	  "Delete all buffers associated with process %s? "
 	  the-procname))
 	(progn
-	  (mapcar '(lambda (buf)
-		     (set-buffer buf)
-		     ;; Consider buffers for which
-		     ;; ess-local-process-name is the same as
-		     ;; the-procname
-		     (if (and (not (get-buffer-process buf))
-			      ess-local-process-name
-			      (equal ess-local-process-name
-				     the-procname))
-			 (kill-buffer buf)))
-		  (buffer-list))
-	  (ess-switch-to-ESS nil)))))
+	  (save-excursion
+	    (mapcar '(lambda (buf)
+		       (set-buffer buf)
+		       ;; Consider buffers for which
+		       ;; ess-local-process-name is the same as
+		       ;; the-procname
+		       (if (and (not (get-buffer-process buf))
+				ess-local-process-name
+				(equal ess-local-process-name
+				       the-procname))
+			   (kill-buffer buf)))
+		    (buffer-list)))))
+    (ess-switch-to-ESS nil)))
 
 (defun ess-kill-buffer-function nil
   "Function run just before an ESS process buffer is killed."
@@ -1877,9 +2004,9 @@ and (indirectly) by \\[ess-get-help-files-list]."
 	    (set-buffer tbuffer)
 	    ;; guaranteed by the initial space:(buffer-disable-undo)
 	    ;; dbg:
-;;- 	    (ess-write-to-dribble-buffer
-;;- 	     (format "(ess-search-list ..): tbuffer %s\n"
-;;- 		     (buffer-name tbuffer)))
+;;-	    (ess-write-to-dribble-buffer
+;;-	     (format "(ess-search-list ..): tbuffer %s\n"
+;;-		     (buffer-name tbuffer)))
 
 	    ;;unnecessary; ess-command does (erase-buffer)
 	    (ess-command inferior-ess-search-list-command tbuffer)
@@ -1894,7 +2021,7 @@ and (indirectly) by \\[ess-get-help-files-list]."
 		    (setq elt (concat homedir elt)))
 		;;else
 		;;dbg
-;;- 		(ess-write-to-dribble-buffer "not dir.\n")
+;;-		(ess-write-to-dribble-buffer "not dir.\n")
 		)
 	      (setq result (append result (list elt))))
 	    (kill-buffer tbuffer)))
