@@ -1,14 +1,13 @@
 ;;; essd-sta.el --- Stata customization
 
 ;; Copyright (C) 1997--1999 A. J. Rossini, Thomas Lumley
+;; Copyright (C) 1997--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
-;; Author: A.J. Rossini <rossini@biostat.washington.edu>
-;; Maintainer: A.J. Rossini <rossini@biostat.washington.edu>
+;; Original Author: A.J. Rossini <rossini@biostat.washington.edu>
 ;; Created: 9 Sep 1998
-;; Modified: $Date: 2001/08/08 19:25:07 $
-;; Version: $Revision: 1.25 $
-;; RCS: $Id: essd-sta.el,v 1.25 2001/08/08 19:25:07 ess Exp $
-;;
+;; Maintainers: ESS-core <ESS-core@stat.math.ethz.ch>
+
 ;; Keywords: start up, configuration.
 
 ;; This file is part of ESS
@@ -48,9 +47,6 @@
     (ess-language                  . "STA")
     (ess-dialect                   . STA-dialect-name)
     (ess-suffix                    . "ado")
-    (ess-dump-filename-template    . (concat (user-login-name)
-					     ".%s."
-					     ess-suffix))
     (ess-mode-editing-alist        . STA-editing-alist)
     (ess-mode-syntax-table         . STA-syntax-table)
     (ess-mode-edit                 . 'STA-mode)
@@ -64,8 +60,8 @@
     (inferior-ess-objects-command  . "description\n")
     (inferior-ess-help-command     . "set more off\n help %s\n")
     (inferior-ess-exit-command     . "exit\n")
-    (inferior-ess-primary-prompt   . "^.") ;; "^. ?") 
-    (inferior-ess-secondary-prompt . "^.") ;; "^. ?") 
+    (inferior-ess-primary-prompt   . "^.") ;; "^. ?")
+    (inferior-ess-secondary-prompt . "^.") ;; "^. ?")
     (comint-use-prompt-regexp-instead-of-fields . t) ;; emacs 21 and up
     (inferior-ess-start-file       . nil) ;"~/.ess-stata")
     (inferior-ess-start-args       . "")) ; "-q"
@@ -90,7 +86,7 @@
 	   ess-dialect
 	   (current-buffer)))
   (let ((sta-start-args
-	 (concat "TERM=emacs stata " 
+	 (concat "TERM=emacs stata "
 	         inferior-ess-start-args
 		 (if start-args (read-string
 				 "Starting Args [possibly -k####] ? ")
