@@ -1,14 +1,13 @@
 ;;; essl-s.el --- Support for editing S source code
 
-;; Copyright (C) 1989-2001 D. Bates, Kademan, Ritter, D.M. Smith, K. Hornik,
-;; R.M. Heiberger, M. Maechler, and A.J. Rossini.
+;; Copyright (C) 1989-1997 D. Bates, Kademan, Ritter, D.M. Smith, K. Hornik,
+;;	R.M. Heiberger, M. Maechler, and A.J. Rossini.
+;; Copyright (C) 1998-2004 A.J. Rossini, Rich M. Heiberger, Martin
+;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
-;; Author: A.J. Rossini <rossini@biostat.washington.edu>
-;; Maintainer: A.J. Rossini <rossini@biostat.washington.edu>
+;; Original Author: A.J. Rossini <rossini@biostat.washington.edu>
 ;; Created: 26 Aug 1997
-;; Modified: $Date: 2002/05/10 19:33:24 $
-;; Version: $Revision: 5.29 $
-;; RCS: $Id: essl-s.el,v 5.29 2002/05/10 19:33:24 rmh Exp $
+;; Maintainers: ESS-core <ESS-core@stat.math.ethz.ch>
 
 ;; This file is part of ESS (Emacs Speaks Statistics).
 
@@ -19,11 +18,11 @@
 
 ;; This file is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
+;; along with GNU Emacs; see the file COPYING.	If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;; Commentary:
@@ -43,57 +42,58 @@
     nil
   (setq S-syntax-table (make-syntax-table))
   (modify-syntax-entry ?\\ "\\" S-syntax-table)
-  (modify-syntax-entry ?+  "."  S-syntax-table)
-  (modify-syntax-entry ?-  "."  S-syntax-table)
-  (modify-syntax-entry ?=  "."  S-syntax-table)
-  (modify-syntax-entry ?%  "."  S-syntax-table)
-  (modify-syntax-entry ?<  "."  S-syntax-table)
-  (modify-syntax-entry ?>  "."  S-syntax-table)
-  (modify-syntax-entry ?&  "."  S-syntax-table)
-  (modify-syntax-entry ?|  "."  S-syntax-table)
+  (modify-syntax-entry ?+  "."	S-syntax-table)
+  (modify-syntax-entry ?-  "."	S-syntax-table)
+  (modify-syntax-entry ?=  "."	S-syntax-table)
+  (modify-syntax-entry ?%  "."	S-syntax-table)
+  (modify-syntax-entry ?<  "."	S-syntax-table)
+  (modify-syntax-entry ?>  "."	S-syntax-table)
+  (modify-syntax-entry ?&  "."	S-syntax-table)
+  (modify-syntax-entry ?|  "."	S-syntax-table)
   (modify-syntax-entry ?\' "\"" S-syntax-table)
-  (modify-syntax-entry ?#  "<"  S-syntax-table) ; open comment
-  (modify-syntax-entry ?\n ">"  S-syntax-table) ; close comment
+  (modify-syntax-entry ?\" "\"" S-syntax-table)
+  (modify-syntax-entry ?#  "<"	S-syntax-table) ; open comment
+  (modify-syntax-entry ?\n ">"	S-syntax-table) ; close comment
   ;;(modify-syntax-entry ?.  "w"  S-syntax-table) ; "." used in S obj names
-  (modify-syntax-entry ?.  "_"  S-syntax-table) ; see above/below,
+  (modify-syntax-entry ?.  "_"	S-syntax-table) ; see above/below,
 					; plus consider separation.
-  (modify-syntax-entry ?$  "_"  S-syntax-table) ; foo.bar$hack is 1 symbol
-  (modify-syntax-entry ?_  "."  S-syntax-table)
-  (modify-syntax-entry ?*  "."  S-syntax-table)
-  (modify-syntax-entry ?<  "."  S-syntax-table)
-  (modify-syntax-entry ?>  "."  S-syntax-table)
-  (modify-syntax-entry ?/  "."  S-syntax-table))
+  (modify-syntax-entry ?$  "_"	S-syntax-table) ; foo.bar$hack is 1 symbol
+  (modify-syntax-entry ?_  "."	S-syntax-table)
+  (modify-syntax-entry ?*  "."	S-syntax-table)
+  (modify-syntax-entry ?<  "."	S-syntax-table)
+  (modify-syntax-entry ?>  "."	S-syntax-table)
+  (modify-syntax-entry ?/  "."	S-syntax-table))
 
 (defvar S-editing-alist
-  '((paragraph-start              . (concat "^$\\|" page-delimiter))
-    (paragraph-separate           . (concat "^$\\|" page-delimiter))
+  '((paragraph-start		  . (concat "\\s-*$\\|" page-delimiter))
+    (paragraph-separate		  . (concat "\\s-*$\\|" page-delimiter))
     (paragraph-ignore-fill-prefix . t)
-    (require-final-newline        . t)
-    (comment-start                . "#")
-    (comment-start-skip           . "#+ *")
-    (comment-column               . 40)
-    ;;(comment-indent-function  . 'S-comment-indent)
-    ;;(ess-comment-indent           . 'S-comment-indent)
-    ;;(ess-indent-line                      . 'S-indent-line)
-    ;;(ess-calculate-indent           . 'S-calculate-indent)
-    (indent-line-function            . 'S-indent-line)
-    (parse-sexp-ignore-comments   . t)
-    (ess-set-style                . ess-default-style)
-    (ess-local-process-name       . nil)
-    ;;(ess-keep-dump-files          . 'ask)
-    (ess-mode-syntax-table        . S-syntax-table)
+    (require-final-newline	  . t)
+    (comment-start		  . "#")
+    (comment-start-skip		  . "#+ *")
+    (comment-column		  . 40)
+    ;;(comment-indent-function	. 'S-comment-indent)
+    ;;(ess-comment-indent	    . 'S-comment-indent)
+    ;;(ess-indent-line			    . 'S-indent-line)
+    ;;(ess-calculate-indent	      . 'S-calculate-indent)
+    (indent-line-function	     . 'S-indent-line)
+    (parse-sexp-ignore-comments	  . t)
+    (ess-set-style		  . ess-default-style)
+    (ess-local-process-name	  . nil)
+    ;;(ess-keep-dump-files	    . 'ask)
+    (ess-mode-syntax-table	  . S-syntax-table)
     ;; For Changelog add, require ' ' before <- : "attr<-" is a function name :
     (add-log-current-defun-header-regexp . "^\\(.+\\)\\s-+<-[ \t\n]*function")
-    (font-lock-defaults           . '(ess-mode-font-lock-keywords
+    (font-lock-defaults		  . '(ess-mode-font-lock-keywords
 				      nil nil ((?\. . "w"))))
     )
   "General options for editing S, S+, and R source files.")
 
 (defvar inferior-S-language-start
   '(concat "options("
-	     "STERM='"  ess-STERM  "'"
+	     "STERM='"	ess-STERM  "'"
 	     (if ess-editor (concat ", editor='" ess-editor "'"))
-	     (if ess-pager  (concat ", pager='"  ess-pager  "', help.pager='"  ess-pager  "'"))
+	     (if ess-pager  (concat ", pager='"	 ess-pager  "', help.pager='"  ess-pager  "'"))
 	     ")")
   "S language expression for startup -- default for all S dialects.")
 
@@ -169,14 +169,14 @@
 (defconst ess-help-S+-sec-regex "^[A-Z. ---]+:$"
   "Reg(ular) Ex(pression) of section headers in help file.")
 
-(defconst ess-help-R-sec-regex "^\\s *[A-Z[a-z. ---]+:$"
+(defconst ess-help-R-sec-regex "^[A-Z][a-z].+:$"
   "Reg(ular) Ex(pression) of section headers in help file.")
 
 ;;; S-mode extras of Martin Maechler, Statistik, ETH Zurich.
 ;;; See also ./ess-utils.el
 
 (defvar ess-function-outline-file
-  (concat ess-lisp-directory "/../etc/" "function-outline.S")
+  (concat ess-etc-directory  "/function-outline.S")
   "The file name of the ess-function outline that is to be inserted at point,
 when \\<ess-mode-map>\\[ess-insert-function-outline] is used.
 Placeholders (substituted `at runtime'): $A$ for `Author', $D$ for `Date'.")
@@ -387,30 +387,19 @@ Uses the file given by the variable `ess-function-outline-file'."
 (defun ess-fix-comments (&optional dont-query verbose)
   "Fix ess-mode buffer so that single-line comments start with at least `##'."
   (interactive "P")
-  (save-excursion
-    (goto-char (point-min))
-    (let ((rgxp "^\\([ \t]*#\\)\\([^#]\\)")
-	  (to   "\\1#\\2"))
-      (if dont-query
-	  (ess-rep-regexp     rgxp to nil nil verbose)
-	(query-replace-regexp rgxp to nil)))))
+  (ess-replace-regexp-dump-to-src "^\\([ \t]*#\\)\\([^#]\\)"
+				  "\\1#\\2" dont-query verbose))
 
 (defun ess-dump-to-src (&optional dont-query verbose)
   "Make the changes in an S - dump() file to improve human readability."
   (interactive "P")
-  (save-excursion
-    (if (not (equal major-mode 'ess-mode))
-	(ess-mode))
-    (goto-char (point-min))
-    (let ((rgxp "^\"\\([a-z.][a-z.0-9]*\\)\"<-\n")
-	  (to   "\n\\1 <- "))
-      (if dont-query
-	  (ess-rep-regexp     rgxp to nil nil verbose)
-	(query-replace-regexp rgxp to nil)))))
+  (ess-replace-regexp-dump-to-src  "^\"\\([a-z.][a-z.0-9]*\\)\"<-\n"
+				   "\n\\1 <- "
+				   dont-query verbose 'ensure-ess))
 
 (defun ess-num-var-round (&optional dont-query verbose)
   "Is VERY useful for dump(.)'ed numeric variables; ROUND some of them by
-  replacing  endings of 000000*.. and 999999*.  Martin Maechler"
+  replacing  endings of 000000*.. and 999999*.	Martin Maechler"
   (interactive "P")
   (save-excursion
     (goto-char (point-min))
@@ -418,7 +407,7 @@ Uses the file given by the variable `ess-function-outline-file'."
     (let ((num 0)
 	  (str "")
 	  (rgxp "000000+[1-9]?[1-9]?\\>")
-	  (to   ""))
+	  (to	""))
       (if dont-query
 	  (ess-rep-regexp     rgxp to nil nil verbose)
 	(query-replace-regexp rgxp to nil))
@@ -432,15 +421,51 @@ Uses the file given by the variable `ess-function-outline-file'."
 			'fixedcase 'literal verbose)
 	(setq num (1+ num))))))
 
+(defun ess-fix-dot (before-chars &optional dont-query verbose)
+  "Remove trailing decimal '.' (\"dot\"), before BEFORE; typically from S-plus"
+  ;; typically, before-chars =	"]:" or more
+  (ess-replace-regexp-dump-to-src
+   (concat "\\([0-9]\\)\\.\\( *[" before-chars "]\\)")
+   ;;		111	 ^
+   "\\1\\2" dont-query verbose))
+
+(defun ess-fix-dot-1 (&optional do-query verbose)
+  "Remove trailing decimal '.' (\"dot\"), before ':' or ']', i.e.,
+in cases where it's ugly and nonsense.	DO-QUERY(prefix) asks before replacing."
+  (interactive "P")
+  (ess-fix-dot "]:" (not do-query) verbose))
+
+(defun ess-fix-dot-more (&optional dont-query verbose)
+  "Remove trailing decimal '.' (\"dot\", typically from S+) in more cases
+ than `ess-fix-dot-1'."
+  (interactive "P")
+  (ess-fix-dot-1 nil verbose)
+  (ess-fix-dot ",)" dont-query verbose))
+
+(defun ess-fix-EQ-assign (&optional dont-query verbose)
+  "Replace \"=\" by \"<-\" at least for function assignments.
+CAREFUL if have list()s of functions!"
+  ;;TODO: "in the few places we can be very sure.."
+  ;;---- is hard in general: local functions: ok; but functions in
+  ;;  list(a = function(x) abs(x), b= function(y) bound(y))  *NOT* ok!
+  (interactive "P")
+  (ess-replace-regexp-dump-to-src
+   "^\\( *[a-z.][_a-z.0-9]*\\) *= *\\(function *(\\)"
+   "\\1 <- \\2" dont-query verbose))
+
 ;;; All of the above three :
 (defun ess-MM-fix-src (&optional dont-query verbose)
-  "Clean up ess-source code which has been produced by  dump(..).
- Produces more readable code, and one that is well formatted in emacs ess-mode."
+  "Clean up ess-source code which has been produced by dump(..), and other
+code not typically produced by other tools.  Produces more readable code,
+and one that is well formatted in emacs ess-mode."
   (interactive "P")
   ;; each of the following does a save-excursion:
   (ess-dump-to-src dont-query)
   (ess-fix-comments dont-query)
-  (ess-num-var-round dont-query verbose))
+  (ess-num-var-round dont-query verbose)
+  (ess-fix-dot-more dont-query verbose)
+  (ess-fix-EQ-assign dont-query verbose)
+  )
 
 (defun ess-fix-miscellaneous (&optional from verbose)
   "Fix Miscellaneous S/R `ill-formation's from current \\[point].
@@ -453,7 +478,9 @@ Uses the file given by the variable `ess-function-outline-file'."
 	 (require 'essd-r)
 	 (R-fix-T-F from (not verbose))))
 
-    (goto-char from) (ess-rep-regexp " *_ *" " <- " nil 'literal verbose)
+    ;;from R 1.9.x "_" is valid in names; here assume no initial / trailing '_'
+    (goto-char from) (ess-rep-regexp " +_ *" " <- " nil 'literal verbose)
+    (goto-char from) (ess-rep-regexp   "_ +" " <- " nil 'literal verbose)
 
     ;; ensure space around  "<-"  ---- but only replace if necessary:
     (goto-char from)
@@ -489,15 +516,33 @@ Uses the file given by the variable `ess-function-outline-file'."
     ))
 
 (defun ess-smart-underscore ()
-  "Electrical \"_\" key: insert `ess-S-assign' after removing spaces, unless
- in string/comment.  `ess-S-assign', typically \" <- \", can be customized."
+  "Smart \"_\" key: insert `ess-S-assign', unless in string/comment.
+If the underscore key is pressed a second time, the assignment
+operator is removed and replaced by the underscore.  `ess-S-assign',
+typically \" <- \", can be customized.	In ESS modes other than R/S,
+an underscore is always inserted. "
   (interactive)
   ;;(insert (if (inside-string/comment-p (point)) "_" ess-S-assign))
-  (if (inside-string/comment-p (point))
+  (if (or
+       (inside-string/comment-p (point))
+       (not (equal ess-language "S")))
       (insert "_")
-    ;; else
-    (delete-horizontal-space)
-    (insert ess-S-assign)))
+    ;; Else one keypress produces ess-S-assign; a second keypress will delete
+    ;; ess-S-assign and instead insert _
+    ;; Rather than trying to count a second _ keypress, just check whether
+    ;; the current point is preceded by ess-S-assign.
+    (let ((assign-len (length ess-S-assign)))
+      (if (and
+	   (>= (point) (+ assign-len (point-min))) ;check that we can move back
+	   (save-excursion
+	     (backward-char assign-len)
+	     (looking-at ess-S-assign)))
+	  ;; If we are currently looking at ess-S-assign, replace it with _
+	  (progn
+	    (delete-backward-char assign-len)
+	    (insert "_"))
+	(delete-horizontal-space)
+	(insert ess-S-assign)))))
 
 (defun ess-toggle-underscore (force)
   "Set the \"_\" (underscore) key to \\[ess-smart-underscore] or back to \"_\".
@@ -513,10 +558,10 @@ Uses the file given by the variable `ess-function-outline-file'."
 	     ;; (stringp uscore) (string= uscore ess-S-assign)
 	     (not force))
 	(progn
-	 (define-key ess-mode-map          "_" nil); 'self-insert-command
+	 (define-key ess-mode-map	   "_" nil); 'self-insert-command
 	 (define-key inferior-ess-mode-map "_" nil))
       ;; else : "force" or uscore is "nil", i.e. default
-      (define-key ess-mode-map          "_" 'ess-smart-underscore)
+      (define-key ess-mode-map		"_" 'ess-smart-underscore)
       (define-key inferior-ess-mode-map "_" 'ess-smart-underscore))))
 
 ;; NOTA BENE: "_" is smart *by default* :
@@ -540,13 +585,11 @@ and I need to relearn emacs lisp (but I had to, anyway."
 
   (interactive "sFunction ? ")
   (let* ((buffname "ess-complete.R")
-	 (buf (ess-execute (format "args(%s)" Sfunc)
-					  t
-					  buffname)))
+	 (buf (ess-execute (format "args(%s)" Sfunc) t buffname)))
     (pop-to-buffer buf)
     (message "here yet?")
     (replace-string "function" Sfunc)
-    (ess-setq-vars-local ess-customize-alist (current-buffer))
+    (ess-setq-vars-local ess-customize-alist); (current-buffer))
     (setq major-mode 'ess-mode)
     (use-local-map ess-mode-map)
     (set-syntax-table ess-mode-syntax-table)
@@ -559,11 +602,11 @@ and I need to relearn emacs lisp (but I had to, anyway."
 
 ;;; This file is automatically placed in Outline minor mode.
 ;;; The file is structured as follows:
-;;; Chapters:     ^L ;
-;;; Sections:    ;;*;;
+;;; Chapters:	  ^L ;
+;;; Sections:	 ;;*;;
 ;;; Subsections: ;;;*;;;
-;;; Components:  defuns, defvars, defconsts
-;;;              Random code beginning with a ;;;;* comment
+;;; Components:	 defuns, defvars, defconsts
+;;;		 Random code beginning with a ;;;;* comment
 
 ;;; Local variables:
 ;;; mode: emacs-lisp
