@@ -1,18 +1,14 @@
 ;;; essl-sas.el --- SAS customization
 
-;; Copyright (C) 1997--2001 Richard M. Heiberger, A. J. Rossini, 
-;;  M. Maechler, Rodney Sparapani
+;; Copyright (C) 1997--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
-;; Authors: Richard M. Heiberger <rmh@astro.ocis.temple.edu>,
+;; Original Authors: Richard M. Heiberger <rmh@astro.ocis.temple.edu>,
 ;;          A.J. Rossini <rossini@u.washington.edu>,
 ;;          Rodney Sparapani <rsparap@mcw.edu>
-;; Maintainer: Richard M. Heiberger <rmh@astro.ocis.temple.edu>,
-;;             Rodney Sparapani <rsparap@mcw.edu>
 ;; Created: 20 Aug 1997
-;; Modified: $Date: 2002/04/12 21:46:24 $
-;; Version: $Revision: 5.42 $
-;; RCS: $Id: essl-sas.el,v 5.42 2002/04/12 21:46:24 rsparapa Exp $
-;;
+;; Maintainers: ESS-core <ESS-core@stat.math.ethz.ch>
+
 ;; Keywords: start up, configuration.
 
 ;; This file is part of ESS (Emacs Speaks Statistics).
@@ -89,7 +85,7 @@ on this mode."
   "Toggle Ess-Listing minor mode.
 With arg, turn Ess-Listing minor mode on if arg is positive, off
 otherwise.  Ess-Listing mode is used solely to place an indicator on
-the mode line." 
+the mode line."
   (interactive "P")
   (setq ess-listing-minor-mode
 	(if (null arg) (not ess-listing-minor-mode)
@@ -113,8 +109,10 @@ the mode line."
   (use-local-map sas-mode-local-map)
   (toggle-read-only t)) ;; to protect the buffer.
 
-(setq auto-mode-alist (append '(("\\.lst\\'" . sas-listing-mode)) auto-mode-alist))
-(setq auto-mode-alist (append '(("\\.LST\\'" . sas-listing-mode)) auto-mode-alist))
+(if (not (featurep 'xemacs))
+    (setq auto-mode-alist
+	(append '(("\\.[lL][sS][tT]\\'" . sas-listing-mode)) auto-mode-alist)))
+;;;###autoload(add-to-list 'auto-mode-alist '("\\.[lL][sS][tT]\\'" . sas-listing-mode))
 
 (fset 'sas-log-mode        'SAS-log-mode)
 (fset 'SAS-transcript-mode 'SAS-log-mode)
@@ -138,7 +136,7 @@ the mode line."
   :type  'boolean)
 
 ;; user can specify the sas program name
-(defcustom sas-program 
+(defcustom sas-program
   (if (equal system-type 'Apple-Macintosh) "invoke SAS using program file" "sas")
   "*Command to invoke SAS, default for buffer-local `ess-sas-submit-command'."
   :group 'ess-sas
@@ -155,7 +153,7 @@ the mode line."
 ;  :group 'ess-sas
 ;  :type  'string)
 
-(defcustom sas-notify t 
+(defcustom sas-notify t
   "*Beep and display message when job is done."
   :group 'ess-sas
   :type  'boolean)
@@ -165,10 +163,10 @@ the mode line."
   :group 'ess-sas
   :type  'boolean)
 
-(defcustom sas-get-options nil 
+(defcustom sas-get-options nil
   "Options to be passed to SAS in sas-get-dataset."
   :group 'ess-sas
-  :type  'string)
+  :type '(choice (const nil) string))
 
 (defcustom sas-get-options-history nil
   "History list of Options passed to SAS in sas-get-dataset."
@@ -185,7 +183,7 @@ number."
   :group 'ess-sas
   :type  'boolean)
 
-(defcustom sas-tmp-libname "_tmp_" 
+(defcustom sas-tmp-libname "_tmp_"
   "*Libname to use for sas-get-dataset."
   :group 'ess-sas
   :type  'string)
@@ -193,7 +191,7 @@ number."
 (defcustom sas-file-name nil
   "*The name of the current sas file."
   :group 'ess-sas
-  :type  'file)
+  :type '(choice (const nil) file))
 
 ;; The next two are ``the inside of [...] in a regexp'' to be used in
 ;; (skip-chars-(for|back)ward SAS-..-chars)
@@ -210,7 +208,7 @@ number."
 (defcustom ess-sas-run-make-regexp nil
   "If you do not want to run make-regexp, then set to nil."
   :group 'ess-sas
-  :type  'string)
+  :type '(choice (const nil) string))
 
 (require 'essa-sas)
 
@@ -252,7 +250,7 @@ number."
       (require 'font-lock)
 
       (defvar SAS-mode-font-lock-keywords
-	(if ess-sas-run-make-regexp 
+	(if ess-sas-run-make-regexp
 	(list
 	 ;; SAS comments
 	 (cons "^[ \t]*%?\\*.*;"		font-lock-comment-face)
@@ -378,7 +376,7 @@ number."
 	 (cons "^ERROR: .*$"                        font-lock-keyword-face)
 
 	 ;; .log WARNING: messages
-	 (cons "^WARNING: .*$"                      font-lock-function-name-face)	
+	 (cons "^WARNING: .*$"                      font-lock-function-name-face)
 
 	 ;; SAS comments
 ;; /* */ handled by grammar above
@@ -395,7 +393,7 @@ number."
 						    font-lock-reference-face)
 
 	 (cons (concat "\\(^[0-9]*\\|;\\|):\\|%then\\|%else\\)[ \t]*"
-		"\\(data\\|endsas\\|quit\\|run\\)[ \t\n;]")
+		"\\(data\\|endsas\\|finish\\|quit\\|run\\|start\\)[ \t\n;]")
 		      				    font-lock-reference-face)
 	 (cons (concat "\\(^[0-9]*\\|;\\|):\\|%then\\|%else\\)[ \t]*"
 		"proc[ \t]+[a-z][a-z_0-9]+")        font-lock-reference-face)
@@ -423,32 +421,40 @@ number."
 						    font-lock-keyword-face)
 
 	 ;; SAS statements
-
 	 (cons (concat
 		"\\(^[0-9]*\\|):\\|[;,]\\|then\\|else\\)[ \t]*"
 		"\\(a\\(bort\\|rray\\|ttrib\\)\\|by"
 		"\\|c\\(hange\\|lass\\|ontrast\\)"
-		"\\|d\\(elete\\|isplay\\|m\\|rop\\)"
+		"\\|d\\(elete\\|isplay\\|m\\|o\\([ \t]+\\(data\\|over\\)\\)?\\|rop\\)"
 		"\\|e\\(rror\\|stimate\\|xc\\(hange\\|lude\\)\\)"
 		"\\|f\\(ile\\(name\\)?\\|o\\(otnote\\(10?\\|[2-9]\\)?\\|rmat\\)\\|req\\)"
 		"\\|go\\([ \t]*to\\|ptions\\)"
 		"\\|i\\(d\\|f\\|n\\(dex\\|f\\(ile\\|ormat\\)\\|put\\|value\\)\\)"
-		"\\|keep\\|l\\(abel\\|ength\\|i\\(bname\\|nk\\)\\|smeans\\)"
+		"\\|keep\\|l\\(abel\\|ength\\|i\\(bname\\|nk\\|st\\)\\|smeans\\)"
 		"\\|m\\(anova\\|e\\(ans\\|rge\\)\\|issing\\|od\\(el\\|ify\\)\\)\\|note"
 		"\\|o\\(ptions\\|therwise\\|utput\\)\\|p\\(lot\\|ut\\)"
 		"\\|r\\(andom\\|e\\(name\\|peated\\|tain\\)\\)"
 		"\\|s\\(ave\\|e\\(lect\\|t\\)\\|kip\\|trata\\|umby\\)"
 		"\\|t\\(ables?\\|i\\(me\\|tle\\(10?\\|[2-9]\\)?\\)\\)\\|update"
-		"\\|va\\(lue\\|r\\)\\|w\\(eight\\|here\\|i\\(ndow\\|th\\)\\)\\)"
+		"\\|va\\(lue\\|r\\)\\|w\\(eight\\|here\\|i\\(ndow\\|th\\)\\)"
+
+	;; IML statements that are not also SAS statements
+		"\\|append\\|c\\(lose\\(file\\)?\\|reate\\)\\|edit\\|f\\(ind\\|orce\\|ree\\)"
+		"\\|insert\\|load\\|mattrib\\|p\\(a[ru]se\\|rint\\|urge\\)"
+		"\\|re\\(move\\|peat\\|place\\|set\\|sume\\)"
+		"\\|s\\(et\\(in\\|out\\)\\|how\\|ort\\|tore\\|ummary\\)\\|use\\)?"
+
 		"\\>")				    font-lock-keyword-face)
-;;	 (cons "\\<\\(\\(\\(then\\|else\\)[ \t]*\\)?do\\([ \t]*over\\)?\\|else\\)\\>"
-	 (cons "\\<\\(\\(then\\|else\\)[ \t]*\\)?\\(do\\([ \t]*over\\)?\\|else\\)\\>"
-						    font-lock-keyword-face)
+
+
+
+;;	 (cons "\\<\\(\\(then\\|else\\)[ \t]*\\)?\\(do\\([ \t]*over\\)?\\|else\\)\\>"
+;;						    font-lock-keyword-face)
 
 	 ;; SAS statements that must be followed by a semi-colon
 	 (cons (concat
 		"\\(^[0-9]*\\|):\\|[;,]\\|then\\|else\\)[ \t]*"
-		"\\(cards4?\\|end\\|l\\(ist\\|ostcard\\)\\|page\\|return\\|stop\\)?"
+		"\\(cards4?\\|datalines\\|end\\|l\\(ostcard\\)\\|page\\|return\\|stop\\)?"
 		"[ \t]*;")			    font-lock-keyword-face)
 
 	 ;; SAS/GRAPH statements not handled above
@@ -457,7 +463,7 @@ number."
 		"\\(axis\\|legend\\|pattern\\|symbol\\)"
 		"\\([1-9][0-9]?\\)?\\>")	    font-lock-keyword-face)
 
-	 ;; SAS functions and SAS macro functions
+	 ;; SAS Datastep functions and SAS macro functions
 	 (cons "%[a-z_][a-z_0-9]*[ \t]*[(;]"
 						    font-lock-function-name-face)
 	 (cons "\\<call[ \t]+[a-z_][a-z_0-9]*[ \t]*("
@@ -489,17 +495,32 @@ number."
 		"\\|c\\(nonct\\|ompbl\\)\\|d\\(airy\\|equote\\)\\|fnonct\\|tnonct"
 		"\\|i\\(bessel\\|n\\(dexw\\|put[cn]\\)\\)\\|jbessel\\|put[cn]"
 		"\\|lowcase\\|quote\\|resolve\\|s\\(oundex\\|ysprod\\)\\|tr\\(anwrd\\|imn\\)"
-		
+
+;;;    ;; IML functions that are not also Datastep functions
+		"\\|a\\(ll\\|ny\\|pply\\|rmasim\\)\\|b\\(lock\\|ranks\\|tran\\)"
+		"\\|c\\(har\\|hoose\\|on\\(cat\\|tents\\|vexit\\|vmod\\)\\|ovlag\\|shape\\|usum\\|vexhull\\)"
+		"\\|d\\(atasets\\|esignf?\\|et\\|iag\\|o\\|uration\\)"
+		"\\|e\\(chelon\\|igv\\(al\\|ec\\)\\)\\|f\\(ft\\|orward\\)\\|ginv"
+		"\\|h\\(alf\\|ankel\\|dir\\|ermite\\|omogen\\)"
+		"\\|i\\(\\|fft\\|nsert\\|nv\\(updt\\)?\\)\\|j\\(\\|root\\)\\|loc\\|mad"
+		"\\|n\\(ame\\|col\\|leng\\|row\\|um\\)\\|o\\(pscal\\|rpol\\)"
+		"\\|p\\(olyroot\\|roduct\\|v\\)\\|r\\(anktie\\|ates\\|atio\\|emove\\|eturn\\|oot\\|owcatc?\\)"
+		"\\|s\\(etdif\\|hape\\|olve\\|plinev\\|pot\\|qrsym\\|ssq\\|torage\\|weep\\|ymsqr\\)"
+		"\\|t\\(\\|eigv\\(al\\|ec\\)\\|oeplitz\\|race\\|risolv\\|ype\\)"
+		"\\|uni\\(on\\|que\\)\\|v\\(alue\\|ecdiag\\)\\|x\\(mult\\|sect\\)\\|yield"
+
 ;;;    ;; SCL functions that are known to work with SAS macro function %sysfunc
 
 		"\\|attr[cn]\\|c\\(exist\\|lose\\)\\|d\\(close\\|num\\|open\\|read\\)"
 		"\\|exist\\|f\\(close\\|etchobs\\|i\\(leexist\\|nfo\\)\\|open\\|put\\|write\\)"
 		"\\|get\\(option\\|var[cn]\\)\\|lib\\(name\\|ref\\)\\|op\\(en\\|t\\(getn\\|setn\\)\\)"
-		"\\|pathname\\|sysmsg\\|var\\(fmt\\|label\\|num\\|type\\)\\)"
+		"\\|pathname\\|sysmsg\\|var\\(fmt\\|l\\(abel\\|en\\)\\|n\\(ame\\|um\\)\\|type\\)\\)"
 		"[ \t]*(")			    font-lock-function-name-face)
 	 ))
 	"Font Lock regexs for SAS.")
-)); only if window-system
+)
+  (defvar SAS-mode-font-lock-keywords "") ;; empty if not window-system
+); only if window-system
 
 (defvar SAS-editing-alist
   '((sentence-end                 . ";[\t\n */]*")
@@ -716,7 +737,7 @@ opening /* appears.  returns 0 otherwise."
       (beginning-of-line)
       (while ok
         (if (re-search-forward
-	     "\\(^[ \t]*run[ ;]\\)\\|\\(^[ \t]*proc \\|^[ \t]*data[ ;]\\)" 
+	     "\\(^[ \t]*run[ ;]\\)\\|\\(^[ \t]*proc \\|^[ \t]*data[ ;]\\)"
 	     nil 1)
 	    (if (match-beginning 2)
 		(if (re-search-forward
@@ -854,7 +875,7 @@ $" "")
   (let ((name (buffer-file-name)))
     (cond ((not name))
           ((string-match (substring name -4 nil)
-			 "\\.sas\\|\\.lst\\|\\.log") 
+			 "\\.sas\\|\\.lst\\|\\.log")
 
            (setq sas-file-name (substring name 0 (- (length name) 4)))
            (setq sas-buffer-name (buffer-name))
